# ----------------------------------------------------------------------------------------------------
# IBM Confidential
# OCO Source Materials
# 5900-A3Q, 5737-J33
# Copyright IBM Corp. 2019
# The source code for this program is not published or other-wise divested of its trade 
# secrets, irrespective of what has been deposited with the U.S.Copyright Office.
# ----------------------------------------------------------------------------------------------------

from utils.assertions import *
from utils.cleanup import *
from utils.waits import *
from utils.wml import *
from utils.wml_deployments.keras import Lstm

from ibm_ai_openscale import APIClient, APIClient4ICP
from ibm_ai_openscale.engines import *
from ibm_ai_openscale.supporting_classes.enums import InputDataType, ProblemType, FeedbackFormat

from random import uniform, randint


class TestAIOpenScaleClient(unittest.TestCase):
    deployment_uid = None

    model_uid = None
    subscription_uid = None
    scoring_url = None
    labels = None
    ai_client = None
    wml_client = None
    binding_uid = None
    subscription = None
    scoring_result = None
    test_uid = str(uuid.uuid4())
    scoring_records = None
    feedback_records = None
    final_run_details = None

    def generate_payload_data(self):
        return [
                    [0.044558, -0.097891, 0.963376],
                    [0.045492, -0.099184, 0.964469],
                    [0.046426, -0.100476, 0.965563],
                    [uniform(0, 1.0), -0.101769, 0.966656],
                    [0.048294, -0.103062, 0.967749],
                    [0.049228, -0.104355, 0.968842],
                    [0.050162, -0.105647, 0.969936],
                    [0.051096, -0.10694, 0.971029],
                    [0.051096, -0.20694, 0.971029],
                    [uniform(0, 1.0), uniform(0, 1.0), uniform(0, 1.0)],
                    [uniform(0, 1.0), uniform(0, 1.0), uniform(0, 1.0)],
                    [0.051092, -0.12694, 0.971029],
                    [0.051796, -0.10694, 0.171129],
                    [0.051096, -0.10694, 0.971029],
                    [uniform(0, 1.0), uniform(0, 1.0), uniform(0, 1.0)],
                    [0.051094, -0.10694, 0.971329],
                    [0.051091, -0.50694, 0.871029],
                    [uniform(0, 1.0), uniform(0, 1.0), uniform(0, 1.0)],
                    [0.049666, -0.105971, 0.970205],
                    [0.048235, -0.105002, 0.96938],
                    [0.046805, -0.104034, 0.968556],
                    [uniform(0, 1.0), uniform(0, 1.0), uniform(0, 1.0)],
                    [0.043944, -0.102096, 0.966908],
                    [0.042514, -0.101127, 0.966083],
                    [0.041083, -0.100158, 0.965259],
                    [0.039653, -0.09919, 0.964435],
                    [0.038222, -0.098221, 0.96361],
                    [0.036792, -0.097252, 0.962786],
                    [uniform(0, 1.0), uniform(0, 1.0), uniform(0, 1.0)],
                    [0.032792, -0.097252, 0.962786],
                    [0.036792, -0.097252, 0.962786],
                    [uniform(0, 1.0), uniform(0, 1.0), uniform(0, 1.0)],
                    [0.036792, -0.097252, 0.962786],
                    [0.031792, -0.097252, 0.962786],
                    [uniform(0, 1.0), uniform(0, 1.0), uniform(0, 1.0)],
                    [0.036592, -0.197252, 0.962786],
                    [0.036792, -0.097252, 0.962786],
                    [uniform(0, 1.0), uniform(0, 1.0), uniform(0, 1.0)],
                    [0.03744, -0.097446, 0.964397],
                    [0.038088, -0.04764, 0.966009],
                    [0.038736, -0.097833, 0.96162],
                    [uniform(0, 1.0), uniform(0, 1.0), uniform(0, 1.0)],
                    [0.040032, -0.098241, 0.975843],
                    [0.04168, -0.098115, 0.972454],
                    [uniform(0, 1.0), uniform(0, 1.0), uniform(0, 1.0)],
                    [0.041976, -0.098802, 0.975676],
                    [0.042624, -0.098996, 0.971288],
                    [0.043272, -0.09919, 0.978299],
                    [uniform(0, 1.0), uniform(0, 1.0), uniform(0, 1.0)],
                    [0.043026, -0.111872, 0.980312],
                    [0.042923, -0.101713, 0.981018],
                    [uniform(0, 1.0), uniform(0, 1.0), uniform(0, 1.0)],
                    [0.042658, -0.103396, 0.98243],
                    [0.044535, -0.114237, 0.983137],
                    [0.042412, -0.105078, 0.983143],
                    [uniform(0, 1.0), uniform(0, 1.0), uniform(0, 1.0)],
                    [uniform(0, 1.0), uniform(0, 1.0), uniform(0, 1.0)],
                    [0.042043, -0.101601, 0.986962],
                    [0.041671, -0.102256, 0.986207],
                    [uniform(0, 1.0), uniform(0, 1.0), uniform(0, 1.0)],
                    [0.032927, -0.102366, 0.983698],
                    [0.036555, -0.140621, 0.912944],
                    [uniform(0, 1.0), uniform(0, 1.0), uniform(0, 1.0)],
                    [0.033812, -0.097131, 0.911434],
                    [0.03244, -0.094386, 0.98268],
                    [0.031068, -0.093641, 0.979925],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.028324, -0.090151, 0.978416],
                    [0.029223, -0.089374, 0.977769],
                    [0.030123, -0.088596, 0.977122],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.032822, -0.086264, 0.97528],
                    [0.033721, -0.084487, 0.974233],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.03152, -0.086933, 0.973238],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.035319, -0.086378, 0.911944],
                    [0.036484, -0.08309, 0.971815],
                    [0.035649, -0.083803, 0.971686],
                    [0.034813, -0.084515, 0.971557],
                    [0.033978, -0.085228, 0.971428],
                    [0.033143, -0.08594, 0.971299],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.031473, -0.087365, 0.911042],
                    [0.030637, -0.089077, 0.910913],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.028967, -0.089502, 0.970655],
                    [0.02975, -0.090297, 0.969868],
                    [0.030532, -0.091052, 0.969081],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.032097, -0.092602, 0.967507],
                    [0.032879, -0.093377, 0.96672],
                    [0.033662, -0.094152, 0.965934],
                    [0.034445, -0.094927, 0.965147],
                    [0.035227, -0.095702, 0.96436],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.036792, -0.097252, 0.962786],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.039104, -0.09531, 0.965501],
                    [0.039875, -0.094663, 0.966406],
                    [0.040646, -0.094016, 0.967311],
                    [0.041417, -0.093368, 0.968216],
                    [0.042188, -0.092721, 0.969121],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.043729, -0.091426, 0.970931],
                    [0.0445, -0.090779, 0.971836],
                    [0.044564, -0.090714, 0.97106],
                    [0.044629, -0.090649, 0.970284],
                    [0.044693, -0.090584, 0.969507],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.044886, -0.090389, 0.967179],
                    [0.04495, -0.090324, 0.966403],
                    [0.045014, -0.090259, 0.965626],
                    [0.045079, -0.090194, 0.96485],
                    [0.045143, -0.090129, 0.964074],
                    [0.044296, -0.089419, 0.965637],
                    [0.043449, -0.088709, 0.9672],
                    [0.042603, -0.087999, 0.968763],
                    [0.041756, -0.087289, 0.970326],
                    [0.040909, -0.086579, 0.971889],
                    [0.040062, -0.085868, 0.973453],
                    [0.039215, -0.085158, 0.975016],
                    [0.038369, -0.084448, 0.976579],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.038111, -0.084708, 0.979683],
                    [0.03883, -0.085548, 0.979673],
                    [0.039548, -0.086388, 0.979662],
                    [0.040266, -0.087228, 0.979651],
                    [0.040984, -0.088068, 0.97964],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.042421, -0.089748, 0.979619],
                    [0.043139, -0.090588, 0.979608],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.04308, -0.091364, 0.979538],
                    [0.042304, -0.0913, 0.979479],
                    [uniform(0, 1.0), uniform(-1.0, 0), uniform(0.5, 1.0)],
                    [0.04075, -0.091173, 0.979361],
                    [0.039974, -0.091109, 0.979302],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.03842, -0.090981, 0.979184],
                    [0.037643, -0.090918, 0.979125],
                    [0.036867, -0.090854, 0.979066],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.037112, -0.089172, 0.977653],
                    [0.038135, -0.087553, 0.9763],
                    [0.039157, -0.085935, 0.974946],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.042224, -0.08108, 0.970885],
                    [0.043246, -0.079461, 0.969532],
                    [0.044268, -0.077843, 0.968178],
                    [0.045291, -0.076224, 0.966825],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.046067, -0.076288, 0.966884],
                    [0.045945, -0.077129, 0.96759],
                    [0.045822, -0.07797, 0.968296],
                    [0.045699, -0.078812, 0.969002],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.045453, -0.080494, 0.970415],
                    [0.045331, -0.081335, 0.971121],
                    [0.045208, -0.082176, 0.971828],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.04425, -0.083729, 0.972405],
                    [0.043415, -0.084442, 0.972276],
                    [0.04258, -0.085154, 0.972147],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.039239, -0.088003, 0.971632],
                    [0.038404, -0.088715, 0.971503],
                    [0.037569, -0.089428, 0.971374],
                    [0.036734, -0.09014, 0.971245],
                    [0.036734, -0.09014, 0.971245],
                    [0.036734, -0.09014, 0.971245],
                    [0.036734, -0.09014, 0.971245],
                    [0.036734, -0.09014, 0.971245],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.036734, -0.09014, 0.971245],
                    [0.036734, -0.09014, 0.971245],
                    [0.037633, -0.089363, 0.970598],
                    [0.038533, -0.088585, 0.969951],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.040332, -0.087031, 0.968656],
                    [0.041231, -0.086253, 0.968009],
                    [0.04213, -0.085476, 0.967362],
                    [0.04303, -0.084699, 0.966715],
                    [0.043929, -0.083922, 0.966067],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.044706, -0.083985, 0.966127],
                    [0.043683, -0.085604, 0.96748],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.040617, -0.090459, 0.971541],
                    [0.039594, -0.092077, 0.972894],
                    [0.038572, -0.093696, 0.974248],
                    [0.03755, -0.095314, 0.975601],
                    [0.036527, -0.096933, 0.976954],
                    [0.035505, -0.098551, 0.978308],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.033333, -0.093897, 0.980878],
                    [0.032609, -0.092346, 0.981735],
                    [0.031885, -0.090795, 0.982592],
                    [0.031162, -0.089244, 0.983449],
                    [0.030438, -0.087693, 0.984306],
                    [0.029714, -0.086141, 0.985162],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.03042, -0.085559, 0.986844],
                    [0.031138, -0.086399, 0.986833],
                    [0.031857, -0.087239, 0.986822],
                    [0.032575, -0.088079, 0.986811],
                    [0.033293, -0.088919, 0.9868],
                    [0.034011, -0.089759, 0.98679],
                    [0.034729, -0.090599, 0.986779],
                    [0.035447, -0.091439, 0.986768],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.047798, -0.09886, 0.971433],
                    [0.050268, -0.100344, 0.968365],
                    [0.052738, -0.101828, 0.965298],
                    [0.055209, -0.103312, 0.962231],
                    [0.057679, -0.104796, 0.959164],
                    [0.060149, -0.10628, 0.956097],
                    [uniform(0, 0.5), uniform(-1.0, 0), uniform(0.5, 1.0)],
                    [0.056563, -0.110811, 0.956994],
                    [0.05477, -0.113077, 0.957443],
                    [0.052977, -0.115343, 0.957891],
                    [0.051184, -0.117609, 0.95834],
                    [0.04939, -0.119874, 0.958789],
                    [0.047597, -0.12214, 0.959237],
                    [0.045804, -0.124406, 0.959686],
                    [0.044011, -0.126671, 0.960134],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.040875, -0.12286, 0.964128],
                    [0.040204, -0.119821, 0.965901],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.038862, -0.113744, 0.969445],
                    [0.03819, -0.110705, 0.971218],
                    [0.037519, -0.107667, 0.97299],
                    [0.036848, -0.104628, 0.974763],
                    [0.036176, -0.10159, 0.976536],
                    [0.035505, -0.098551, 0.978308],
                    [0.03477, -0.095577, 0.980857],
                    [0.034034, -0.092604, 0.983405],
                    [0.033299, -0.08963, 0.985954],
                    [0.032563, -0.086657, 0.988503],
                    [0.031828, -0.083683, 0.991051],
                    [0.031092, -0.080709, 0.9936],
                    [0.030357, -0.077736, 0.996149],
                    [0.029621, -0.074762, 0.998698],
                    [0.028886, -0.071789, 1.001246],
                    [0.02815, -0.068815, 1.003795],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.032879, -0.077733, 0.993998],
                    [0.033667, -0.079219, 0.992365],
                    [0.034456, -0.080705, 0.990732],
                    [0.035244, -0.082192, 0.989099],
                    [0.036032, -0.083678, 0.987466],
                    [0.036768, -0.086652, 0.984917],
                    [0.037503, -0.089625, 0.982369],
                    [0.038239, -0.092599, 0.97982],
                    [0.038974, -0.095572, 0.977272],
                    [0.03971, -0.098546, 0.974723],
                    [0.040446, -0.10152, 0.972174],
                    [0.041181, -0.104493, 0.969626],
                    [0.041917, -0.107467, 0.967077],
                    [0.042652, -0.11044, 0.964529],
                    [0.043388, -0.113414, 0.96198],
                    [0.044059, -0.116453, 0.960207],
                    [0.04473, -0.119491, 0.958435],
                    [0.045402, -0.12253, 0.956662],
                    [0.046073, -0.125568, 0.95489],
                    [0.046744, -0.128607, 0.953117],
                    [0.047415, -0.131645, 0.951345],
                    [0.048086, -0.134684, 0.949573],
                    [0.048758, -0.137722, 0.9478],
                    [0.049429, -0.140761, 0.946028],
                    [0.0501, -0.143799, 0.944255],
                    [0.050153, -0.142312, 0.945171],
                    [0.050205, -0.140824, 0.946086],
                    [0.050258, -0.139337, 0.947002],
                    [0.050311, -0.13785, 0.947918],
                    [0.050363, -0.136363, 0.948834],
                    [0.050416, -0.134875, 0.949749],
                    [0.050469, -0.133388, 0.950665],
                    [0.050522, -0.131901, 0.951581],
                    [0.050574, -0.130413, 0.952496],
                    [0.050627, -0.128926, 0.953412],
                    [0.050026, -0.126534, 0.953562],
                    [0.049425, -0.124141, 0.953713],
                    [0.048824, -0.121749, 0.953863],
                    [0.048223, -0.119356, 0.954014],
                    [0.047621, -0.116964, 0.954164],
                    [0.04702, -0.114572, 0.954315],
                    [0.046419, -0.112179, 0.954465],
                    [0.045818, -0.109787, 0.954616],
                    [0.045217, -0.107394, 0.954766],
                    [0.044616, -0.105002, 0.954917],
                    [0.092385, -0.09331, 0.959854],
                    [0.019634, -0.05328, 0.9257],
                    [0.04333, -0.106302, 0.970439],
                    [0.031424, -0.072679, 0.956529],
                    [0.047073, -0.08818, 0.940791],
                    [0.030137, -0.073978, 0.972052],
                    [0.031424, -0.072679, 0.956529],
                    [0.044384, -0.076555, 0.988754],
                    [0.030021, -0.059754, 0.98897],
                    [0.030137, -0.073978, 0.972052],
                    [0.027681, -0.090801, 0.986178],
                    [0.019518, -0.039056, 0.942619],
                    [0.045438, -0.046808, 1.00707],
                    [0.032478, -0.042932, 0.974844],
                    [0.044384, -0.076555, 0.988754],
                    [0.044267, -0.062331, 1.005673],
                    [0.045786, -0.089479, 0.956313],
                    [0.063543, -0.045486, 0.977205],
                    [0.046957, -0.073956, 0.95771],
                    [0.015891, -0.071402, 0.955349],
                    [0.028967, -0.089502, 0.970655],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.04567, -0.075255, 0.973232],
                    [0.042043, -0.107601, 0.985962],
                    [0.013551, -0.102448, 0.952556],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.026511, -0.106324, 0.984781],
                    [0.0445, -0.090779, 0.971836],
                    [-0.009352, -0.151639, 0.993753],
                    [-0.023714, -0.134839, 0.993969],
                    [0.003957, -0.198186, 0.975222],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.029876, -0.205938, 1.039673],
                    [0.004894, -0.154216, 1.010456],
                    [0.01563, -0.203362, 1.02297],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.010862, -0.090823, 1.00052],
                    [0.021946, -0.182641, 0.962277],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.037246, -0.15547, 0.997295],
                    [0.014837, -0.101149, 0.937033],
                    [0.013435, -0.088224, 0.969475],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.014605, -0.072701, 0.970871],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.032478, -0.042932, 0.974844],
                    [0.03271, -0.07138, 0.941007],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.000475, -0.084349, 0.937249],
                    [0.018232, -0.040355, 0.958142],
                    [0.015891, -0.071402, 0.955349],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.024199, 0.023037, 0.948205],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.005039, -0.008032, 0.959754],
                    [0.020804, -0.037757, 0.927097],
                    [0.037275, 0.004937, 0.963511],
                    [0.036337, -0.039034, 0.928277],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.036337, -0.039034, 0.928277],
                    [0.023145, -0.00671, 0.92989],
                    [0.06483, -0.044187, 0.961683],
                    [0.020572, -0.009309, 0.960934],
                    [0.002931, -0.067526, 0.923123],
                    [0.014721, -0.086925, 0.953952],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.001761, -0.083049, 0.921727],
                    [0.026627, -0.120548, 0.967862],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.011211, -0.133495, 0.949763],
                    [0.016124, -0.09985, 0.921511],
                    [-0.21502, -0.153159, 1.008056],
                    [-0.583132, -0.218178, 0.814726],
                    [-1.359114, -0.244959, 1.062929],
                    [-2.326845, -0.230519, 1.188526],
                    [-3.418833, 0.137113, 0.640924],
                    [-2.819625, -0.092826, 1.820201],
                    [-2.746278, -0.359092, 1.394806],
                    [-3.227748, 0.68462, 0.167506],
                    [-2.484653, -0.052002, 0.560877],
                    [-1.856968, -0.417403, 0.598048],
                    [-0.31234, 0.374499, 0.996947],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [-0.610419, 0.120667, 0.919519],
                    [-1.937482, -0.205246, 0.231427],
                    [0.223668, -0.010388, 0.977676],
                    [1.271346, -0.320981, 0.928342],
                    [0.935123, -0.515575, 1.623866],
                    [-0.209307, -0.063965, 1.236326],
                    [-0.548214, -0.374596, 0.599935],
                    [0.202168, -0.072503, 0.986433],
                    [0.243038, -0.010412, 1.136658],
                    [0.168646, -0.086771, 0.998198],
                    [0.037923, -0.243459, 1.10015],
                    [-0.088186, -0.160777, 0.986455],
                    [-0.153122, -0.12982, 1.046616],
                    [-0.173853, -0.111579, 0.511338],
                    [0.015238, -0.165982, 1.482464],
                    [-0.176934, -0.062574, 1.018796],
                    [-0.13158, -0.062413, 0.629122],
                    [0.032964, -0.097181, 0.7028],
                    [0.018115, -0.026132, 0.97506],
                    [0.012969, -0.031329, 1.03715],
                    [0.109436, -0.121736, 0.911675],
                    [0.092501, -0.107534, 0.942936],
                    [0.043562, -0.13475, 0.936601],
                    [0.04216, -0.121825, 0.969043],
                    [0.043562, -0.13475, 0.936601],
                    [0.023, -0.152894, 0.980592],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.057692, -0.123102, 0.970223],
                    [0.057576, -0.108878, 0.987142],
                    [0.072171, -0.154126, 0.953088],
                    [0.040757, -0.108901, 1.001484],
                    [0.057576, -0.108878, 0.987142],
                    [0.027913, -0.119249, 0.95234],
                    [0.0292, -0.117949, 0.936817],
                    [0.027797, -0.105025, 0.969259],
                    [0.011094, -0.119271, 0.966682],
                    [0.014837, -0.101149, 0.937033],
                    [0.045786, -0.089479, 0.956313],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.042043, -0.107601, 0.985962],
                    [0.060033, -0.092056, 0.973016],
                    [0.013435, -0.088224, 0.969475],
                    [0.028967, -0.089502, 0.970655],
                    [0.017294, -0.084326, 0.922907],
                    [0.000475, -0.084349, 0.937249],
                    [-0.031877, -0.083094, 0.950411],
                    [-0.048696, -0.083116, 0.964753],
                    [-0.095295, -0.079285, 0.961212],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [-0.144466, -0.078052, 0.988715],
                    [-0.159882, -0.090999, 0.970616],
                    [-0.174245, -0.074199, 0.970832],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [-0.222129, -0.071667, 0.982813],
                    [-0.267441, -0.066536, 0.963749],
                    [-0.282857, -0.079483, 0.94565],
                    [-0.298507, -0.063982, 0.961389],
                    [-0.298507, -0.063982, 0.961389],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [-0.364264, -0.09122, 0.969396],
                    [-0.37617, -0.057597, 0.955486],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [-0.37734, -0.07312, 0.95409],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [-0.269781, -0.097583, 0.960957],
                    [-0.238832, -0.085913, 0.980236],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [-0.088535, -0.118105, 1.037212],
                    [-0.009701, -0.108968, 1.04451],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.035611, -0.114098, 1.063574],
                    [0.095285, -0.136029, 1.082422],
                    [0.100198, -0.102384, 1.05417],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.161043, -0.108792, 1.074414],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.151943, -0.101017, 0.995621],
                    [0.165019, -0.119117, 1.010927],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.21033, -0.124248, 1.029991],
                    [0.167592, -0.116518, 0.979883],
                    [0.136526, -0.113964, 0.977522],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.106747, -0.110111, 0.959638],
                    [0.092385, -0.09331, 0.959854],
                    [0.091214, -0.108834, 0.958458],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.032827, -0.085603, 0.924088],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.001878, -0.097273, 0.904808],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [-0.04355, -0.077919, 0.902663],
                    [-0.060253, -0.092165, 0.900086],
                    [-0.043434, -0.092142, 0.885744],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [-0.059083, -0.076642, 0.901483],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [-0.028018, -0.079196, 0.903844],
                    [0.000475, -0.084349, 0.937249],
                    [0.014837, -0.101149, 0.937033],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.045786, -0.089479, 0.956313],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.074395, -0.108856, 0.9728],
                    [0.04567, -0.075255, 0.973232],
                    [0.106631, -0.095887, 0.976557],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.089928, -0.110133, 0.97398],
                    [0.087355, -0.112732, 1.005025],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.073109, -0.110155, 0.988322],
                    [0.074279, -0.094632, 0.989719],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.05746, -0.094654, 1.004061],
                    [0.058746, -0.093355, 0.988538],
                    [0.027681, -0.090801, 0.986178],
                    [0.027681, -0.090801, 0.986178],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.010978, -0.105047, 0.983601],
                    [0.013435, -0.088224, 0.969475],
                    [-0.002098, -0.086947, 0.968294],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [-0.001982, -0.101171, 0.951375],
                    [-0.016344, -0.084371, 0.951591],
                    [-0.017515, -0.099894, 0.950195],
                    [-0.002098, -0.086947, 0.968294],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.014837, -0.101149, 0.937033],
                    [0.028967, -0.089502, 0.970655],
                    [0.028967, -0.089502, 0.970655],
                    [0.028967, -0.089502, 0.970655],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.028967, -0.089502, 0.970655],
                    [0.041927, -0.093377, 1.00288],
                    [0.040757, -0.108901, 1.001484],
                    [0.058746, -0.093355, 0.988538],
                    [0.058746, -0.093355, 0.988538],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.05746, -0.094654, 1.004061],
                    [0.058863, -0.107579, 0.97162],
                    [0.057576, -0.108878, 0.987142],
                    [0.060033, -0.092056, 0.973016],
                    [0.058863, -0.107579, 0.97162],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.04333, -0.106302, 0.970439],
                    [0.045786, -0.089479, 0.956313],
                    [0.047073, -0.08818, 0.940791],
                    [0.0445, -0.090779, 0.971836],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.014721, -0.086925, 0.953952],
                    [0.013435, -0.088224, 0.969475],
                    [0.016007, -0.085626, 0.93843],
                    [0.012264, -0.103748, 0.968078],
                    [0.014721, -0.086925, 0.953952],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.013551, -0.102448, 0.952556],
                    [0.014721, -0.086925, 0.953952],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.027681, -0.090801, 0.986178],
                    [0.027797, -0.105025, 0.969259],
                    [0.012264, -0.103748, 0.968078],
                    [0.013435, -0.088224, 0.969475],
                    [0.027681, -0.090801, 0.986178],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.042043, -0.107601, 0.985962],
                    [0.040757, -0.108901, 1.001484],
                    [0.025224, -0.107624, 1.000304],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.042043, -0.107601, 0.985962],
                    [0.045786, -0.089479, 0.956313],
                    [0.028967, -0.089502, 0.970655],
                    [0.0445, -0.090779, 0.971836],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.030137, -0.073978, 0.972052],
                    [0.047073, -0.08818, 0.940791],
                    [0.0445, -0.090779, 0.971836],
                    [0.03037, -0.102426, 0.938214],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.016007, -0.085626, 0.93843],
                    [0.014721, -0.086925, 0.953952],
                    [0.030254, -0.088202, 0.955133],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.013435, -0.088224, 0.969475],
                    [0.030254, -0.088202, 0.955133],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.027681, -0.090801, 0.986178],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.027797, -0.105025, 0.969259],
                    [0.027797, -0.105025, 0.969259],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.042043, -0.107601, 0.985962],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.027797, -0.105025, 0.969259],
                    [0.045786, -0.089479, 0.956313],
                    [0.044616, -0.105002, 0.954917],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.027681, -0.090801, 0.986178],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.015891, -0.071402, 0.955349],
                    [0.030254, -0.088202, 0.955133],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.0445, -0.090779, 0.971836],
                    [0.04567, -0.075255, 0.973232],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.04333, -0.106302, 0.970439],
                    [0.028967, -0.089502, 0.970655],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.027797, -0.105025, 0.969259],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.028967, -0.089502, 0.970655],
                    [0.027681, -0.090801, 0.986178],
                    [0.013435, -0.088224, 0.969475],
                    [0.0445, -0.090779, 0.971836],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.027681, -0.090801, 0.986178],
                    [0.027797, -0.105025, 0.969259],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.028851, -0.075278, 0.987574],
                    [0.043214, -0.092078, 0.987358],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.029084, -0.103725, 0.953736],
                    [0.043214, -0.092078, 0.987358],
                    [0.0445, -0.090779, 0.971836],
                    [0.0445, -0.090779, 0.971836],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.045786, -0.089479, 0.956313],
                    [0.028967, -0.089502, 0.970655],
                    [0.028967, -0.089502, 0.970655],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.029084, -0.103725, 0.953736],
                    [0.014721, -0.086925, 0.953952],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.028967, -0.089502, 0.970655],
                    [0.009691, -0.106346, 0.999123],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.030254, -0.088202, 0.955133],
                    [0.027681, -0.090801, 0.986178],
                    [0.027797, -0.105025, 0.969259],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.028967, -0.089502, 0.970655],
                    [0.026511, -0.106324, 0.984781],
                    [0.027681, -0.090801, 0.986178],
                    [0.014721, -0.086925, 0.953952],
                    [0.029084, -0.103725, 0.953736],
                    [uniform(0, 0.1), uniform(-1.0, 0), uniform(0.5, 1.0)],
                    [0.013435, -0.088224, 0.969475],
                    [uniform(0, 0.1), uniform(-1.0, 0), uniform(0.5, 1.0)],
                    [uniform(0, 0.1), uniform(-1.0, 0), uniform(0.5, 1.0)],
                    [uniform(0, 0.1), uniform(-1.0, 0), uniform(0.5, 1.0)],
                    [uniform(0, 0.1), uniform(-1.0, 0), uniform(0.5, 1.0)],
                    [0.027797, -0.105025, 0.969259],
                    [0.047073, -0.08818, 0.940791],
                    [0.013435, -0.088224, 0.969475],
                    [0.029084, -0.103725, 0.953736],
                    [0.042043, -0.107601, 0.985962],
                    [0.030254, -0.088202, 0.955133],
                    [0.028967, -0.089502, 0.970655],
                    [0.028967, -0.089502, 0.970655],
                    [0.030254, -0.088202, 0.955133],
                    [0.027681, -0.090801, 0.986178],
                    [0.04567, -0.075255, 0.973232],
                    [0.013435, -0.088224, 0.969475],
                    [0.027797, -0.105025, 0.969259],
                    [uniform(0, 0.1), uniform(-1.0, 0), uniform(0.5, 1.0)],
                    [0.043214, -0.092078, 0.987358],
                    [0.027681, -0.090801, 0.986178],
                    [uniform(0, 0.1), uniform(-1.0, 0), uniform(0.5, 1.0)],
                    [0.028967, -0.089502, 0.970655],
                    [0.04333, -0.106302, 0.970439],
                    [0.029084, -0.103725, 0.953736],
                    [0.030137, -0.073978, 0.972052],
                    [0.02534, -0.121847, 0.983385],
                    [0.042043, -0.107601, 0.985962],
                    [0.0445, -0.090779, 0.971836],
                    [0.014605, -0.072701, 0.970871],
                    [0.013435, -0.088224, 0.969475],
                    [0.0445, -0.090779, 0.971836],
                    [0.027797, -0.105025, 0.969259],
                    [0.043214, -0.092078, 0.987358],
                    [0.028967, -0.089502, 0.970655],
                    [0.030137, -0.073978, 0.972052],
                    [0.030254, -0.088202, 0.955133],
                    [0.030254, -0.088202, 0.955133],
                    [0.012148, -0.089524, 0.984997],
                    [0.026511, -0.106324, 0.984781],
                    [uniform(0, 0.1), uniform(-1.0, 0), uniform(0.5, 1.0)],
                    [0.028967, -0.089502, 0.970655],
                    [0.028967, -0.089502, 0.970655],
                    [uniform(0, 0.1), uniform(-1.0, 0), uniform(0.5, 1.0)],
                    [0.012264, -0.103748, 0.968078],
                    [0.027797, -0.105025, 0.969259],
                    [0.014721, -0.086925, 0.953952],
                    [0.030254, -0.088202, 0.955133],
                    [0.026511, -0.106324, 0.984781],
                    [0.0445, -0.090779, 0.971836],
                    [0.030254, -0.088202, 0.955133],
                    [0.028967, -0.089502, 0.970655],
                    [0.027681, -0.090801, 0.986178],
                    [0.028967, -0.089502, 0.970655],
                    [0.0445, -0.090779, 0.971836],
                    [0.012148, -0.089524, 0.984997],
                    [0.012264, -0.103748, 0.968078],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.028967, -0.089502, 0.970655],
                    [0.027797, -0.105025, 0.969259],
                    [0.030254, -0.088202, 0.955133],
                    [0.0445, -0.090779, 0.971836],
                    [0.027797, -0.105025, 0.969259],
                    [0.028967, -0.089502, 0.970655],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.028967, -0.089502, 0.970655],
                    [0.012148, -0.089524, 0.984997],
                    [0.027797, -0.105025, 0.969259],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.031424, -0.072679, 0.956529],
                    [0.030254, -0.088202, 0.955133],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.027681, -0.090801, 0.986178],
                    [0.028967, -0.089502, 0.970655],
                    [0.028967, -0.089502, 0.970655],
                    [0.027797, -0.105025, 0.969259],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [uniform(-1.0, 1.0), uniform(-1.0, 1.0), uniform(-1.0, 1.0)],
                    [0.045786, -0.089479, 0.956313],
                    [0.027681, -0.090801, 0.986178],
                    [0.028967, -0.089502, 0.970655],
                    [0.0445, -0.090779, 0.971836],
                    [0.0445, -0.090779, 0.971836]
                ]

    @classmethod
    def setUpClass(cls):
        cls.schema = get_schema_name()
        cls.aios_credentials = get_aios_credentials()
        cls.database_credentials = get_database_credentials()
        cls.deployment = Lstm()

        if is_icp():
            cls.ai_client = APIClient4ICP(cls.aios_credentials)
        else:
            cls.ai_client = APIClient(cls.aios_credentials)
            cls.wml_credentials = get_wml_credentials()



    def test_01_setup_data_mart(self):
        self.ai_client.data_mart.setup(db_credentials=self.database_credentials, schema=self.schema)

    def test_02_bind_wml_instance(self):
        if is_icp():
            TestAIOpenScaleClient.binding_uid = self.ai_client.data_mart.bindings.add("WML instance on ICP",
                                                                                      WatsonMachineLearningInstance4ICP())
        else:
            TestAIOpenScaleClient.binding_uid = self.ai_client.data_mart.bindings.add("WML instance on Cloud",
                                                                                      WatsonMachineLearningInstance(
                                                                                          self.wml_credentials))

    def test_03_get_model_ids(self):
        TestAIOpenScaleClient.model_uid = self.deployment.get_asset_id()
        TestAIOpenScaleClient.deployment_uid = self.deployment.get_deployment_id()



    def test_06_subscribe(self):
        subscription = TestAIOpenScaleClient.ai_client.data_mart.subscriptions.add(
            WatsonMachineLearningAsset(
                source_uid=self.model_uid,
                binding_uid=self.binding_uid,
                problem_type=ProblemType.BINARY_CLASSIFICATION,
                input_data_type=InputDataType.UNSTRUCTURED_TEXT,
                label_column='label',
                prediction_column='prediction_classes',
                probability_column='probability'
            )
        )
        TestAIOpenScaleClient.subscription_uid = subscription.uid
        print("Subscription details: {}".format(subscription.get_details()))

    def test_07_select_asset_and_get_details(self):
        TestAIOpenScaleClient.subscription = TestAIOpenScaleClient.ai_client.data_mart.subscriptions.get(self.subscription_uid)
        print("Subscription details: {}".format(self.subscription.get_details()))

    def test_08_list_deployments(self):
        TestAIOpenScaleClient.subscription.list_deployments()

    def test_09_validate_default_subscription_configuration(self):
        subscription_details = TestAIOpenScaleClient.subscription.get_details()
        assert_monitors_enablement(subscription_details=subscription_details, payload=True, performance=True)

    def test_10_get_payload_logging_details(self):
        TestAIOpenScaleClient.subscription.payload_logging.get_details()

    def test_11_get_performance_monitor_details(self):
        TestAIOpenScaleClient.subscription.performance_monitoring.get_details()

    def test_12_score(self):
        scoring_payload = {"values":
            [
                [
                    [0.044558, -0.097891, 0.963376],
                    [0.045492, -0.099184, 0.964469],
                    [0.046426, -0.100476, 0.965563],
                    [0.04736, -0.101769, 0.966656],
                    [0.048294, -0.103062, 0.967749],
                    [0.049228, -0.104355, 0.968842],
                    [0.050162, -0.105647, 0.969936],
                    [0.051096, -0.10694, 0.971029],
                    [0.051096, -0.10694, 0.971029],
                    [0.051096, -0.10694, 0.971029],
                    [0.051096, -0.10694, 0.971029],
                    [0.051096, -0.10694, 0.971029],
                    [0.051096, -0.10694, 0.971029],
                    [0.051096, -0.10694, 0.971029],
                    [0.051096, -0.10694, 0.971029],
                    [0.051096, -0.10694, 0.971029],
                    [0.051096, -0.10694, 0.971029],
                    [0.051096, -0.10694, 0.971029],
                    [0.049666, -0.105971, 0.970205],
                    [0.048235, -0.105002, 0.96938],
                    [0.046805, -0.104034, 0.968556],
                    [0.045374, -0.103065, 0.967732],
                    [0.043944, -0.102096, 0.966908],
                    [0.042514, -0.101127, 0.966083],
                    [0.041083, -0.100158, 0.965259],
                    [0.039653, -0.09919, 0.964435],
                    [0.038222, -0.098221, 0.96361],
                    [0.036792, -0.097252, 0.962786],
                    [0.036792, -0.097252, 0.962786],
                    [0.036792, -0.097252, 0.962786],
                    [0.036792, -0.097252, 0.962786],
                    [0.036792, -0.097252, 0.962786],
                    [0.036792, -0.097252, 0.962786],
                    [0.036792, -0.097252, 0.962786],
                    [0.036792, -0.097252, 0.962786],
                    [0.036792, -0.097252, 0.962786],
                    [0.036792, -0.097252, 0.962786],
                    [0.036792, -0.097252, 0.962786],
                    [0.03744, -0.097446, 0.964397],
                    [0.038088, -0.09764, 0.966009],
                    [0.038736, -0.097833, 0.96762],
                    [0.039384, -0.098027, 0.969231],
                    [0.040032, -0.098221, 0.970843],
                    [0.04068, -0.098415, 0.972454],
                    [0.041328, -0.098609, 0.974065],
                    [0.041976, -0.098802, 0.975676],
                    [0.042624, -0.098996, 0.977288],
                    [0.043272, -0.09919, 0.978899],
                    [0.043149, -0.100031, 0.979605],
                    [0.043026, -0.100872, 0.980312],
                    [0.042903, -0.101713, 0.981018],
                    [0.04278, -0.102554, 0.981724],
                    [0.042658, -0.103396, 0.98243],
                    [0.042535, -0.104237, 0.983137],
                    [0.042412, -0.105078, 0.983843],
                    [0.042289, -0.105919, 0.984549],
                    [0.042166, -0.10676, 0.985256],
                    [0.042043, -0.107601, 0.985962],
                    [0.040671, -0.105856, 0.985207],
                    [0.039299, -0.104111, 0.984453],
                    [0.037927, -0.102366, 0.983698],
                    [0.036555, -0.100621, 0.982944],
                    [0.035183, -0.098876, 0.982189],
                    [0.033812, -0.097131, 0.981434],
                    [0.03244, -0.095386, 0.98068],
                    [0.031068, -0.093641, 0.979925],
                    [0.029696, -0.091896, 0.979171],
                    [0.028324, -0.090151, 0.978416],
                    [0.029223, -0.089374, 0.977769],
                    [0.030123, -0.088596, 0.977122],
                    [0.031023, -0.087819, 0.976474],
                    [0.031922, -0.087042, 0.975827],
                    [0.032822, -0.086264, 0.97518],
                    [0.033721, -0.085487, 0.974533],
                    [0.03462, -0.08471, 0.973886],
                    [0.03552, -0.083933, 0.973238],
                    [0.03642, -0.083155, 0.972591],
                    [0.037319, -0.082378, 0.971944],
                    [0.036484, -0.08309, 0.971815],
                    [0.035649, -0.083803, 0.971686],
                    [0.034813, -0.084515, 0.971557],
                    [0.033978, -0.085228, 0.971428],
                    [0.033143, -0.08594, 0.971299],
                    [0.032308, -0.086652, 0.971171],
                    [0.031473, -0.087365, 0.971042],
                    [0.030637, -0.088077, 0.970913],
                    [0.029802, -0.08879, 0.970784],
                    [0.028967, -0.089502, 0.970655],
                    [0.02975, -0.090277, 0.969868],
                    [0.030532, -0.091052, 0.969081],
                    [0.031315, -0.091827, 0.968294],
                    [0.032097, -0.092602, 0.967507],
                    [0.032879, -0.093377, 0.96672],
                    [0.033662, -0.094152, 0.965934],
                    [0.034445, -0.094927, 0.965147],
                    [0.035227, -0.095702, 0.96436],
                    [0.03601, -0.096477, 0.963573],
                    [0.036792, -0.097252, 0.962786],
                    [0.037563, -0.096605, 0.963691],
                    [0.038334, -0.095957, 0.964596],
                    [0.039104, -0.09531, 0.965501],
                    [0.039875, -0.094663, 0.966406],
                    [0.040646, -0.094016, 0.967311],
                    [0.041417, -0.093368, 0.968216],
                    [0.042188, -0.092721, 0.969121],
                    [0.042958, -0.092074, 0.970026],
                    [0.043729, -0.091426, 0.970931],
                    [0.0445, -0.090779, 0.971836],
                    [0.044564, -0.090714, 0.97106],
                    [0.044629, -0.090649, 0.970284],
                    [0.044693, -0.090584, 0.969507],
                    [0.044757, -0.090519, 0.968731],
                    [0.044822, -0.090454, 0.967955],
                    [0.044886, -0.090389, 0.967179],
                    [0.04495, -0.090324, 0.966403],
                    [0.045014, -0.090259, 0.965626],
                    [0.045079, -0.090194, 0.96485],
                    [0.045143, -0.090129, 0.964074],
                    [0.044296, -0.089419, 0.965637],
                    [0.043449, -0.088709, 0.9672],
                    [0.042603, -0.087999, 0.968763],
                    [0.041756, -0.087289, 0.970326],
                    [0.040909, -0.086579, 0.971889],
                    [0.040062, -0.085868, 0.973453],
                    [0.039215, -0.085158, 0.975016],
                    [0.038369, -0.084448, 0.976579],
                    [0.037522, -0.083738, 0.978142],
                    [0.036675, -0.083028, 0.979705],
                    [0.037393, -0.083868, 0.979694],
                    [0.038111, -0.084708, 0.979683],
                    [0.03883, -0.085548, 0.979673],
                    [0.039548, -0.086388, 0.979662],
                    [0.040266, -0.087228, 0.979651],
                    [0.040984, -0.088068, 0.97964],
                    [0.041702, -0.088908, 0.979629],
                    [0.042421, -0.089748, 0.979619],
                    [0.043139, -0.090588, 0.979608],
                    [0.043857, -0.091428, 0.979597],
                    [0.04308, -0.091364, 0.979538],
                    [0.042304, -0.0913, 0.979479],
                    [0.041527, -0.091237, 0.97942],
                    [0.04075, -0.091173, 0.979361],
                    [0.039974, -0.091109, 0.979302],
                    [0.039197, -0.091045, 0.979243],
                    [0.03842, -0.090981, 0.979184],
                    [0.037643, -0.090918, 0.979125],
                    [0.036867, -0.090854, 0.979066],
                    [0.03609, -0.09079, 0.979007],
                    [0.037112, -0.089172, 0.977653],
                    [0.038135, -0.087553, 0.9763],
                    [0.039157, -0.085935, 0.974946],
                    [0.040179, -0.084316, 0.973593],
                    [0.041202, -0.082698, 0.972239],
                    [0.042224, -0.08108, 0.970885],
                    [0.043246, -0.079461, 0.969532],
                    [0.044268, -0.077843, 0.968178],
                    [0.045291, -0.076224, 0.966825],
                    [0.046313, -0.074606, 0.965471],
                    [0.04619, -0.075447, 0.966177],
                    [0.046067, -0.076288, 0.966884],
                    [0.045945, -0.077129, 0.96759],
                    [0.045822, -0.07797, 0.968296],
                    [0.045699, -0.078812, 0.969002],
                    [0.045576, -0.079653, 0.969709],
                    [0.045453, -0.080494, 0.970415],
                    [0.045331, -0.081335, 0.971121],
                    [0.045208, -0.082176, 0.971828],
                    [0.045085, -0.083017, 0.972534],
                    [0.04425, -0.083729, 0.972405],
                    [0.043415, -0.084442, 0.972276],
                    [0.04258, -0.085154, 0.972147],
                    [0.041745, -0.085866, 0.972018],
                    [0.04091, -0.086579, 0.971889],
                    [0.040074, -0.087291, 0.971761],
                    [0.039239, -0.088003, 0.971632],
                    [0.038404, -0.088715, 0.971503],
                    [0.037569, -0.089428, 0.971374],
                    [0.036734, -0.09014, 0.971245],
                    [0.036734, -0.09014, 0.971245],
                    [0.036734, -0.09014, 0.971245],
                    [0.036734, -0.09014, 0.971245],
                    [0.036734, -0.09014, 0.971245],
                    [0.036734, -0.09014, 0.971245],
                    [0.036734, -0.09014, 0.971245],
                    [0.036734, -0.09014, 0.971245],
                    [0.036734, -0.09014, 0.971245],
                    [0.036734, -0.09014, 0.971245],
                    [0.036734, -0.09014, 0.971245],
                    [0.037633, -0.089363, 0.970598],
                    [0.038533, -0.088585, 0.969951],
                    [0.039432, -0.087808, 0.969303],
                    [0.040332, -0.087031, 0.968656],
                    [0.041231, -0.086253, 0.968009],
                    [0.04213, -0.085476, 0.967362],
                    [0.04303, -0.084699, 0.966715],
                    [0.043929, -0.083922, 0.966067],
                    [0.044829, -0.083144, 0.96542],
                    [0.045728, -0.082367, 0.964773],
                    [0.044706, -0.083985, 0.966127],
                    [0.043683, -0.085604, 0.96748],
                    [0.042661, -0.087222, 0.968834],
                    [0.041639, -0.088841, 0.970187],
                    [0.040617, -0.090459, 0.971541],
                    [0.039594, -0.092077, 0.972894],
                    [0.038572, -0.093696, 0.974248],
                    [0.03755, -0.095314, 0.975601],
                    [0.036527, -0.096933, 0.976954],
                    [0.035505, -0.098551, 0.978308],
                    [0.034781, -0.097, 0.979165],
                    [0.034057, -0.095449, 0.980022],
                    [0.033333, -0.093897, 0.980878],
                    [0.032609, -0.092346, 0.981735],
                    [0.031885, -0.090795, 0.982592],
                    [0.031162, -0.089244, 0.983449],
                    [0.030438, -0.087693, 0.984306],
                    [0.029714, -0.086141, 0.985162],
                    [0.02899, -0.08459, 0.986019],
                    [0.028266, -0.083039, 0.986876],
                    [0.028984, -0.083879, 0.986865],
                    [0.029702, -0.084719, 0.986854],
                    [0.03042, -0.085559, 0.986844],
                    [0.031138, -0.086399, 0.986833],
                    [0.031857, -0.087239, 0.986822],
                    [0.032575, -0.088079, 0.986811],
                    [0.033293, -0.088919, 0.9868],
                    [0.034011, -0.089759, 0.98679],
                    [0.034729, -0.090599, 0.986779],
                    [0.035447, -0.091439, 0.986768],
                    [0.037917, -0.092923, 0.983701],
                    [0.040387, -0.094407, 0.980634],
                    [0.042858, -0.095891, 0.977567],
                    [0.045328, -0.097375, 0.9745],
                    [0.047798, -0.09886, 0.971433],
                    [0.050268, -0.100344, 0.968365],
                    [0.052738, -0.101828, 0.965298],
                    [0.055209, -0.103312, 0.962231],
                    [0.057679, -0.104796, 0.959164],
                    [0.060149, -0.10628, 0.956097],
                    [0.058356, -0.108546, 0.956546],
                    [0.056563, -0.110811, 0.956994],
                    [0.05477, -0.113077, 0.957443],
                    [0.052977, -0.115343, 0.957891],
                    [0.051184, -0.117609, 0.95834],
                    [0.04939, -0.119874, 0.958789],
                    [0.047597, -0.12214, 0.959237],
                    [0.045804, -0.124406, 0.959686],
                    [0.044011, -0.126671, 0.960134],
                    [0.042218, -0.128937, 0.960583],
                    [0.041547, -0.125898, 0.962356],
                    [0.040875, -0.12286, 0.964128],
                    [0.040204, -0.119821, 0.965901],
                    [0.039533, -0.116783, 0.967673],
                    [0.038862, -0.113744, 0.969445],
                    [0.03819, -0.110705, 0.971218],
                    [0.037519, -0.107667, 0.97299],
                    [0.036848, -0.104628, 0.974763],
                    [0.036176, -0.10159, 0.976536],
                    [0.035505, -0.098551, 0.978308],
                    [0.03477, -0.095577, 0.980857],
                    [0.034034, -0.092604, 0.983405],
                    [0.033299, -0.08963, 0.985954],
                    [0.032563, -0.086657, 0.988503],
                    [0.031828, -0.083683, 0.991051],
                    [0.031092, -0.080709, 0.9936],
                    [0.030357, -0.077736, 0.996149],
                    [0.029621, -0.074762, 0.998698],
                    [0.028886, -0.071789, 1.001246],
                    [0.02815, -0.068815, 1.003795],
                    [0.028938, -0.070301, 1.002162],
                    [0.029726, -0.071788, 1.000529],
                    [0.030515, -0.073274, 0.998896],
                    [0.031303, -0.07476, 0.997263],
                    [0.032091, -0.076246, 0.99563],
                    [0.032879, -0.077733, 0.993998],
                    [0.033667, -0.079219, 0.992365],
                    [0.034456, -0.080705, 0.990732],
                    [0.035244, -0.082192, 0.989099],
                    [0.036032, -0.083678, 0.987466],
                    [0.036768, -0.086652, 0.984917],
                    [0.037503, -0.089625, 0.982369],
                    [0.038239, -0.092599, 0.97982],
                    [0.038974, -0.095572, 0.977272],
                    [0.03971, -0.098546, 0.974723],
                    [0.040446, -0.10152, 0.972174],
                    [0.041181, -0.104493, 0.969626],
                    [0.041917, -0.107467, 0.967077],
                    [0.042652, -0.11044, 0.964529],
                    [0.043388, -0.113414, 0.96198],
                    [0.044059, -0.116453, 0.960207],
                    [0.04473, -0.119491, 0.958435],
                    [0.045402, -0.12253, 0.956662],
                    [0.046073, -0.125568, 0.95489],
                    [0.046744, -0.128607, 0.953117],
                    [0.047415, -0.131645, 0.951345],
                    [0.048086, -0.134684, 0.949573],
                    [0.048758, -0.137722, 0.9478],
                    [0.049429, -0.140761, 0.946028],
                    [0.0501, -0.143799, 0.944255],
                    [0.050153, -0.142312, 0.945171],
                    [0.050205, -0.140824, 0.946086],
                    [0.050258, -0.139337, 0.947002],
                    [0.050311, -0.13785, 0.947918],
                    [0.050363, -0.136363, 0.948834],
                    [0.050416, -0.134875, 0.949749],
                    [0.050469, -0.133388, 0.950665],
                    [0.050522, -0.131901, 0.951581],
                    [0.050574, -0.130413, 0.952496],
                    [0.050627, -0.128926, 0.953412],
                    [0.050026, -0.126534, 0.953562],
                    [0.049425, -0.124141, 0.953713],
                    [0.048824, -0.121749, 0.953863],
                    [0.048223, -0.119356, 0.954014],
                    [0.047621, -0.116964, 0.954164],
                    [0.04702, -0.114572, 0.954315],
                    [0.046419, -0.112179, 0.954465],
                    [0.045818, -0.109787, 0.954616],
                    [0.045217, -0.107394, 0.954766],
                    [0.044616, -0.105002, 0.954917],
                    [0.092385, -0.09331, 0.959854],
                    [0.019634, -0.05328, 0.9257],
                    [0.04333, -0.106302, 0.970439],
                    [0.031424, -0.072679, 0.956529],
                    [0.047073, -0.08818, 0.940791],
                    [0.030137, -0.073978, 0.972052],
                    [0.031424, -0.072679, 0.956529],
                    [0.044384, -0.076555, 0.988754],
                    [0.030021, -0.059754, 0.98897],
                    [0.030137, -0.073978, 0.972052],
                    [0.027681, -0.090801, 0.986178],
                    [0.019518, -0.039056, 0.942619],
                    [0.045438, -0.046808, 1.00707],
                    [0.032478, -0.042932, 0.974844],
                    [0.044384, -0.076555, 0.988754],
                    [0.044267, -0.062331, 1.005673],
                    [0.045786, -0.089479, 0.956313],
                    [0.063543, -0.045486, 0.977205],
                    [0.046957, -0.073956, 0.95771],
                    [0.015891, -0.071402, 0.955349],
                    [0.028967, -0.089502, 0.970655],
                    [0.017061, -0.055879, 0.956745],
                    [0.028967, -0.089502, 0.970655],
                    [0.04567, -0.075255, 0.973232],
                    [0.042043, -0.107601, 0.985962],
                    [0.013551, -0.102448, 0.952556],
                    [0.027797, -0.105025, 0.969259],
                    [0.028967, -0.089502, 0.970655],
                    [0.026511, -0.106324, 0.984781],
                    [0.0445, -0.090779, 0.971836],
                    [-0.009352, -0.151639, 0.993753],
                    [-0.023714, -0.134839, 0.993969],
                    [0.003957, -0.198186, 0.975222],
                    [0.006297, -0.16714, 0.978015],
                    [0.029876, -0.205938, 1.039673],
                    [0.004894, -0.154216, 1.010456],
                    [0.01563, -0.203362, 1.02297],
                    [0.020311, -0.141269, 1.028556],
                    [0.019373, -0.18524, 0.993321],
                    [0.010862, -0.090823, 1.00052],
                    [0.021946, -0.182641, 0.962277],
                    [0.016007, -0.085626, 0.93843],
                    [0.037246, -0.15547, 0.997295],
                    [0.014837, -0.101149, 0.937033],
                    [0.013435, -0.088224, 0.969475],
                    [0.056173, -0.095954, 1.019583],
                    [0.040757, -0.108901, 1.001484],
                    [0.014605, -0.072701, 0.970871],
                    [0.05746, -0.094654, 1.004061],
                    [0.032478, -0.042932, 0.974844],
                    [0.03271, -0.07138, 0.941007],
                    [0.049297, -0.04291, 0.960502],
                    [0.000475, -0.084349, 0.937249],
                    [0.018232, -0.040355, 0.958142],
                    [0.015891, -0.071402, 0.955349],
                    [0.021742, 0.006214, 0.962331],
                    [0.058746, -0.093355, 0.988538],
                    [0.024199, 0.023037, 0.948205],
                    [-0.003501, -0.074023, 1.000736],
                    [0.005039, -0.008032, 0.959754],
                    [0.020804, -0.037757, 0.927097],
                    [0.037275, 0.004937, 0.963511],
                    [0.036337, -0.039034, 0.928277],
                    [-0.027197, -0.021001, 0.955997],
                    [0.005155, -0.022256, 0.942835],
                    [0.036337, -0.039034, 0.928277],
                    [0.023145, -0.00671, 0.92989],
                    [0.06483, -0.044187, 0.961683],
                    [0.020572, -0.009309, 0.960934],
                    [0.002931, -0.067526, 0.923123],
                    [0.014721, -0.086925, 0.953952],
                    [0.014721, -0.086925, 0.953952],
                    [0.001761, -0.083049, 0.921727],
                    [0.026627, -0.120548, 0.967862],
                    [0.011094, -0.119271, 0.966682],
                    [0.011211, -0.133495, 0.949763],
                    [0.016124, -0.09985, 0.921511],
                    [-0.21502, -0.153159, 1.008056],
                    [-0.583132, -0.218178, 0.814726],
                    [-1.359114, -0.244959, 1.062929],
                    [-2.326845, -0.230519, 1.188526],
                    [-3.418833, 0.137113, 0.640924],
                    [-2.819625, -0.092826, 1.820201],
                    [-2.746278, -0.359092, 1.394806],
                    [-3.227748, 0.68462, 0.167506],
                    [-2.484653, -0.052002, 0.560877],
                    [-1.856968, -0.417403, 0.598048],
                    [-0.31234, 0.374499, 0.996947],
                    [0.294904, -0.864339, 2.013745],
                    [-0.610419, 0.120667, 0.919519],
                    [-1.937482, -0.205246, 0.231427],
                    [0.223668, -0.010388, 0.977676],
                    [1.271346, -0.320981, 0.928342],
                    [0.935123, -0.515575, 1.623866],
                    [-0.209307, -0.063965, 1.236326],
                    [-0.548214, -0.374596, 0.599935],
                    [0.202168, -0.072503, 0.986433],
                    [0.243038, -0.010412, 1.136658],
                    [0.168646, -0.086771, 0.998198],
                    [0.037923, -0.243459, 1.10015],
                    [-0.088186, -0.160777, 0.986455],
                    [-0.153122, -0.12982, 1.046616],
                    [-0.173853, -0.111579, 0.511338],
                    [0.015238, -0.165982, 1.482464],
                    [-0.176934, -0.062574, 1.018796],
                    [-0.13158, -0.062413, 0.629122],
                    [0.032964, -0.097181, 0.7028],
                    [0.018115, -0.026132, 0.97506],
                    [0.012969, -0.031329, 1.03715],
                    [0.109436, -0.121736, 0.911675],
                    [0.092501, -0.107534, 0.942936],
                    [0.043562, -0.13475, 0.936601],
                    [0.04216, -0.121825, 0.969043],
                    [0.043562, -0.13475, 0.936601],
                    [0.023, -0.152894, 0.980592],
                    [0.039703, -0.138648, 0.983169],
                    [0.057692, -0.123102, 0.970223],
                    [0.057576, -0.108878, 0.987142],
                    [0.072171, -0.154126, 0.953088],
                    [0.040757, -0.108901, 1.001484],
                    [0.057576, -0.108878, 0.987142],
                    [0.027913, -0.119249, 0.95234],
                    [0.0292, -0.117949, 0.936817],
                    [0.027797, -0.105025, 0.969259],
                    [0.011094, -0.119271, 0.966682],
                    [0.014837, -0.101149, 0.937033],
                    [0.045786, -0.089479, 0.956313],
                    [0.04333, -0.106302, 0.970439],
                    [0.042043, -0.107601, 0.985962],
                    [0.060033, -0.092056, 0.973016],
                    [0.013435, -0.088224, 0.969475],
                    [0.028967, -0.089502, 0.970655],
                    [0.017294, -0.084326, 0.922907],
                    [0.000475, -0.084349, 0.937249],
                    [-0.031877, -0.083094, 0.950411],
                    [-0.048696, -0.083116, 0.964753],
                    [-0.095295, -0.079285, 0.961212],
                    [-0.095295, -0.079285, 0.961212],
                    [-0.144466, -0.078052, 0.988715],
                    [-0.159882, -0.090999, 0.970616],
                    [-0.174245, -0.074199, 0.970832],
                    [-0.174245, -0.074199, 0.970832],
                    [-0.222129, -0.071667, 0.982813],
                    [-0.267441, -0.066536, 0.963749],
                    [-0.282857, -0.079483, 0.94565],
                    [-0.298507, -0.063982, 0.961389],
                    [-0.298507, -0.063982, 0.961389],
                    [-0.315326, -0.064005, 0.975731],
                    [-0.342648, -0.043329, 0.943721],
                    [-0.364264, -0.09122, 0.969396],
                    [-0.37617, -0.057597, 0.955486],
                    [-0.378627, -0.07442, 0.969612],
                    [-0.37734, -0.07312, 0.95409],
                    [-0.378627, -0.07442, 0.969612],
                    [-0.347561, -0.076974, 0.971973],
                    [-0.269781, -0.097583, 0.960957],
                    [-0.238832, -0.085913, 0.980236],
                    [-0.196094, -0.093643, 1.030345],
                    [-0.149495, -0.097474, 1.033886],
                    [-0.088535, -0.118105, 1.037212],
                    [-0.009701, -0.108968, 1.04451],
                    [0.021248, -0.097298, 1.06379],
                    [0.035611, -0.114098, 1.063574],
                    [0.095285, -0.136029, 1.082422],
                    [0.100198, -0.102384, 1.05417],
                    [0.163616, -0.106193, 1.043369],
                    [0.161043, -0.108792, 1.074414],
                    [0.145743, -0.135962, 1.039396],
                    [0.151943, -0.101017, 0.995621],
                    [0.165019, -0.119117, 1.010927],
                    [0.183008, -0.103572, 0.997982],
                    [0.21033, -0.124248, 1.029991],
                    [0.167592, -0.116518, 0.979883],
                    [0.136526, -0.113964, 0.977522],
                    [0.136526, -0.113964, 0.977522],
                    [0.122164, -0.097164, 0.977738],
                    [0.106747, -0.110111, 0.959638],
                    [0.092385, -0.09331, 0.959854],
                    [0.091214, -0.108834, 0.958458],
                    [0.048359, -0.08688, 0.925268],
                    [0.032827, -0.085603, 0.924088],
                    [0.03154, -0.086903, 0.93961],
                    [0.001878, -0.097273, 0.904808],
                    [-0.027901, -0.09342, 0.886925],
                    [-0.026731, -0.077896, 0.888321],
                    [-0.04355, -0.077919, 0.902663],
                    [-0.060253, -0.092165, 0.900086],
                    [-0.043434, -0.092142, 0.885744],
                    [-0.029304, -0.080495, 0.919366],
                    [-0.059083, -0.076642, 0.901483],
                    [-0.028018, -0.079196, 0.903844],
                    [-0.028018, -0.079196, 0.903844],
                    [0.000475, -0.084349, 0.937249],
                    [0.014837, -0.101149, 0.937033],
                    [0.029084, -0.103725, 0.953736],
                    [0.029084, -0.103725, 0.953736],
                    [0.045786, -0.089479, 0.956313],
                    [0.058746, -0.093355, 0.988538],
                    [0.074395, -0.108856, 0.9728],
                    [0.04567, -0.075255, 0.973232],
                    [0.106631, -0.095887, 0.976557],
                    [0.105344, -0.097186, 0.99208],
                    [0.089928, -0.110133, 0.97398],
                    [0.087355, -0.112732, 1.005025],
                    [0.088525, -0.097209, 1.006422],
                    [0.073109, -0.110155, 0.988322],
                    [0.074279, -0.094632, 0.989719],
                    [0.074395, -0.108856, 0.9728],
                    [0.05746, -0.094654, 1.004061],
                    [0.058746, -0.093355, 0.988538],
                    [0.027681, -0.090801, 0.986178],
                    [0.027681, -0.090801, 0.986178],
                    [0.042043, -0.107601, 0.985962],
                    [0.010978, -0.105047, 0.983601],
                    [0.013435, -0.088224, 0.969475],
                    [-0.002098, -0.086947, 0.968294],
                    [-0.002098, -0.086947, 0.968294],
                    [-0.001982, -0.101171, 0.951375],
                    [-0.016344, -0.084371, 0.951591],
                    [-0.017515, -0.099894, 0.950195],
                    [-0.002098, -0.086947, 0.968294],
                    [-0.001982, -0.101171, 0.951375],
                    [0.014837, -0.101149, 0.937033],
                    [0.028967, -0.089502, 0.970655],
                    [0.028967, -0.089502, 0.970655],
                    [0.028967, -0.089502, 0.970655],
                    [0.029084, -0.103725, 0.953736],
                    [0.028967, -0.089502, 0.970655],
                    [0.041927, -0.093377, 1.00288],
                    [0.040757, -0.108901, 1.001484],
                    [0.058746, -0.093355, 0.988538],
                    [0.058746, -0.093355, 0.988538],
                    [0.058863, -0.107579, 0.97162],
                    [0.05746, -0.094654, 1.004061],
                    [0.058863, -0.107579, 0.97162],
                    [0.057576, -0.108878, 0.987142],
                    [0.060033, -0.092056, 0.973016],
                    [0.058863, -0.107579, 0.97162],
                    [0.043214, -0.092078, 0.987358],
                    [0.04333, -0.106302, 0.970439],
                    [0.045786, -0.089479, 0.956313],
                    [0.047073, -0.08818, 0.940791],
                    [0.0445, -0.090779, 0.971836],
                    [0.027797, -0.105025, 0.969259],
                    [0.014721, -0.086925, 0.953952],
                    [0.013435, -0.088224, 0.969475],
                    [0.016007, -0.085626, 0.93843],
                    [0.012264, -0.103748, 0.968078],
                    [0.014721, -0.086925, 0.953952],
                    [0.014721, -0.086925, 0.953952],
                    [0.013551, -0.102448, 0.952556],
                    [0.014721, -0.086925, 0.953952],
                    [0.013435, -0.088224, 0.969475],
                    [0.027681, -0.090801, 0.986178],
                    [0.027797, -0.105025, 0.969259],
                    [0.012264, -0.103748, 0.968078],
                    [0.013435, -0.088224, 0.969475],
                    [0.027681, -0.090801, 0.986178],
                    [0.04333, -0.106302, 0.970439],
                    [0.042043, -0.107601, 0.985962],
                    [0.040757, -0.108901, 1.001484],
                    [0.025224, -0.107624, 1.000304],
                    [0.027681, -0.090801, 0.986178],
                    [0.044616, -0.105002, 0.954917],
                    [0.042043, -0.107601, 0.985962],
                    [0.045786, -0.089479, 0.956313],
                    [0.028967, -0.089502, 0.970655],
                    [0.0445, -0.090779, 0.971836],
                    [0.03154, -0.086903, 0.93961],
                    [0.030137, -0.073978, 0.972052],
                    [0.047073, -0.08818, 0.940791],
                    [0.0445, -0.090779, 0.971836],
                    [0.03037, -0.102426, 0.938214],
                    [0.015891, -0.071402, 0.955349],
                    [0.016007, -0.085626, 0.93843],
                    [0.014721, -0.086925, 0.953952],
                    [0.030254, -0.088202, 0.955133],
                    [0.030137, -0.073978, 0.972052],
                    [0.013435, -0.088224, 0.969475],
                    [0.030254, -0.088202, 0.955133],
                    [0.026511, -0.106324, 0.984781],
                    [0.027681, -0.090801, 0.986178],
                    [0.028967, -0.089502, 0.970655],
                    [0.027797, -0.105025, 0.969259],
                    [0.027797, -0.105025, 0.969259],
                    [0.027797, -0.105025, 0.969259],
                    [0.030254, -0.088202, 0.955133],
                    [0.042043, -0.107601, 0.985962],
                    [0.027681, -0.090801, 0.986178],
                    [0.044616, -0.105002, 0.954917],
                    [0.027681, -0.090801, 0.986178],
                    [0.027797, -0.105025, 0.969259],
                    [0.045786, -0.089479, 0.956313],
                    [0.044616, -0.105002, 0.954917],
                    [0.027681, -0.090801, 0.986178],
                    [0.030254, -0.088202, 0.955133],
                    [0.027681, -0.090801, 0.986178],
                    [0.012264, -0.103748, 0.968078],
                    [0.015891, -0.071402, 0.955349],
                    [0.030254, -0.088202, 0.955133],
                    [0.013435, -0.088224, 0.969475],
                    [0.0445, -0.090779, 0.971836],
                    [0.04567, -0.075255, 0.973232],
                    [0.015891, -0.071402, 0.955349],
                    [0.045786, -0.089479, 0.956313],
                    [0.04333, -0.106302, 0.970439],
                    [0.028967, -0.089502, 0.970655],
                    [0.027797, -0.105025, 0.969259],
                    [0.028967, -0.089502, 0.970655],
                    [0.027797, -0.105025, 0.969259],
                    [0.0445, -0.090779, 0.971836],
                    [0.028967, -0.089502, 0.970655],
                    [0.028967, -0.089502, 0.970655],
                    [0.027681, -0.090801, 0.986178],
                    [0.013435, -0.088224, 0.969475],
                    [0.0445, -0.090779, 0.971836],
                    [0.028967, -0.089502, 0.970655],
                    [0.013435, -0.088224, 0.969475],
                    [0.027681, -0.090801, 0.986178],
                    [0.027797, -0.105025, 0.969259],
                    [0.029084, -0.103725, 0.953736],
                    [0.028851, -0.075278, 0.987574],
                    [0.043214, -0.092078, 0.987358],
                    [0.013435, -0.088224, 0.969475],
                    [0.029084, -0.103725, 0.953736],
                    [0.043214, -0.092078, 0.987358],
                    [0.0445, -0.090779, 0.971836],
                    [0.0445, -0.090779, 0.971836],
                    [0.04333, -0.106302, 0.970439],
                    [0.045786, -0.089479, 0.956313],
                    [0.028967, -0.089502, 0.970655],
                    [0.028967, -0.089502, 0.970655],
                    [0.029084, -0.103725, 0.953736],
                    [0.029084, -0.103725, 0.953736],
                    [0.014721, -0.086925, 0.953952],
                    [0.013435, -0.088224, 0.969475],
                    [0.028967, -0.089502, 0.970655],
                    [0.009691, -0.106346, 0.999123],
                    [0.0445, -0.090779, 0.971836],
                    [0.030254, -0.088202, 0.955133],
                    [0.027681, -0.090801, 0.986178],
                    [0.027797, -0.105025, 0.969259],
                    [0.026511, -0.106324, 0.984781],
                    [0.028967, -0.089502, 0.970655],
                    [0.026511, -0.106324, 0.984781],
                    [0.027681, -0.090801, 0.986178],
                    [0.014721, -0.086925, 0.953952],
                    [0.029084, -0.103725, 0.953736],
                    [0.028967, -0.089502, 0.970655],
                    [0.013435, -0.088224, 0.969475],
                    [0.029084, -0.103725, 0.953736],
                    [0.029084, -0.103725, 0.953736],
                    [0.013435, -0.088224, 0.969475],
                    [0.028967, -0.089502, 0.970655],
                    [0.027797, -0.105025, 0.969259],
                    [0.047073, -0.08818, 0.940791],
                    [0.013435, -0.088224, 0.969475],
                    [0.029084, -0.103725, 0.953736],
                    [0.042043, -0.107601, 0.985962],
                    [0.030254, -0.088202, 0.955133],
                    [0.028967, -0.089502, 0.970655],
                    [0.028967, -0.089502, 0.970655],
                    [0.030254, -0.088202, 0.955133],
                    [0.027681, -0.090801, 0.986178],
                    [0.04567, -0.075255, 0.973232],
                    [0.013435, -0.088224, 0.969475],
                    [0.027797, -0.105025, 0.969259],
                    [0.028967, -0.089502, 0.970655],
                    [0.043214, -0.092078, 0.987358],
                    [0.027681, -0.090801, 0.986178],
                    [0.044616, -0.105002, 0.954917],
                    [0.028967, -0.089502, 0.970655],
                    [0.04333, -0.106302, 0.970439],
                    [0.029084, -0.103725, 0.953736],
                    [0.030137, -0.073978, 0.972052],
                    [0.02534, -0.121847, 0.983385],
                    [0.042043, -0.107601, 0.985962],
                    [0.0445, -0.090779, 0.971836],
                    [0.014605, -0.072701, 0.970871],
                    [0.013435, -0.088224, 0.969475],
                    [0.0445, -0.090779, 0.971836],
                    [0.027797, -0.105025, 0.969259],
                    [0.043214, -0.092078, 0.987358],
                    [0.028967, -0.089502, 0.970655],
                    [0.030137, -0.073978, 0.972052],
                    [0.030254, -0.088202, 0.955133],
                    [0.030254, -0.088202, 0.955133],
                    [0.012148, -0.089524, 0.984997],
                    [0.026511, -0.106324, 0.984781],
                    [0.043214, -0.092078, 0.987358],
                    [0.028967, -0.089502, 0.970655],
                    [0.028967, -0.089502, 0.970655],
                    [0.028967, -0.089502, 0.970655],
                    [0.012264, -0.103748, 0.968078],
                    [0.027797, -0.105025, 0.969259],
                    [0.014721, -0.086925, 0.953952],
                    [0.030254, -0.088202, 0.955133],
                    [0.026511, -0.106324, 0.984781],
                    [0.0445, -0.090779, 0.971836],
                    [0.030254, -0.088202, 0.955133],
                    [0.028967, -0.089502, 0.970655],
                    [0.027681, -0.090801, 0.986178],
                    [0.028967, -0.089502, 0.970655],
                    [0.0445, -0.090779, 0.971836],
                    [0.012148, -0.089524, 0.984997],
                    [0.012264, -0.103748, 0.968078],
                    [0.027681, -0.090801, 0.986178],
                    [0.028967, -0.089502, 0.970655],
                    [0.027797, -0.105025, 0.969259],
                    [0.030254, -0.088202, 0.955133],
                    [0.0445, -0.090779, 0.971836],
                    [0.027797, -0.105025, 0.969259],
                    [0.028967, -0.089502, 0.970655],
                    [0.013435, -0.088224, 0.969475],
                    [0.028967, -0.089502, 0.970655],
                    [0.012148, -0.089524, 0.984997],
                    [0.027797, -0.105025, 0.969259],
                    [0.030254, -0.088202, 0.955133],
                    [0.031424, -0.072679, 0.956529],
                    [0.030254, -0.088202, 0.955133],
                    [0.043214, -0.092078, 0.987358],
                    [0.027681, -0.090801, 0.986178],
                    [0.028967, -0.089502, 0.970655],
                    [0.028967, -0.089502, 0.970655],
                    [0.027797, -0.105025, 0.969259],
                    [0.0445, -0.090779, 0.971836],
                    [0.028967, -0.089502, 0.970655],
                    [0.045786, -0.089479, 0.956313],
                    [0.027681, -0.090801, 0.986178],
                    [0.028967, -0.089502, 0.970655],
                    [0.0445, -0.090779, 0.971836],
                    [0.0445, -0.090779, 0.971836]
                ],
                self.generate_payload_data(),
                self.generate_payload_data(),
                self.generate_payload_data(),
                self.generate_payload_data(),
                self.generate_payload_data(),
                self.generate_payload_data(),
                self.generate_payload_data(),
                self.generate_payload_data()
            ]
        }

        scoring_requests = 5
        TestAIOpenScaleClient.scoring_records = scoring_requests * len(scoring_payload['values'])
        for i in range(0, scoring_requests):
            scores = self.deployment.score(payload=scoring_payload)
            self.assertIsNotNone(scores)

        print("Scoring result: {}".format(scores))

        wait_for_payload_table(subscription=self.subscription, payload_records=self.scoring_records)

    def test_13_stats_on_payload_logging_table(self):
        self.subscription.payload_logging.print_table_schema()
        assert_payload_logging_unstructured_data(subscription=self.subscription, scoring_records=self.scoring_records)

    def test_14_stats_on_performance_monitoring_table(self):
        if is_icp():
            self.skipTest("Performance monitoring is not working on ICP with WML scoring.")

        wait_for_performance_table(subscription=self.subscription)

        self.subscription.performance_monitoring.print_table_schema()
        self.subscription.performance_monitoring.show_table()
        self.subscription.performance_monitoring.describe_table()

        performance_table_pandas = self.subscription.performance_monitoring.get_table_content()
        assert_performance_monitoring_pandas_table_content(pandas_table_content=performance_table_pandas)

        performance_table_python = self.subscription.performance_monitoring.get_table_content(format='python')
        assert_performance_monitoring_python_table_content(python_table_content=performance_table_python)

    def test_15_enable_quality_monitoring(self):
        self.subscription.quality_monitoring.enable(threshold=0.7, min_records=10)

        details = TestAIOpenScaleClient.subscription.quality_monitoring.get_details()
        assert_quality_monitoring_configuration(quality_monitoring_details=details)

    def test_16_feedback_logging(self):
        TestAIOpenScaleClient.feedback_records = 20
        feedback_data = []
        for i in range(0, self.feedback_records):
            feedback_data.append([self.generate_payload_data(), [randint(0, 1)]])

        self.subscription.feedback_logging.store(feedback_data=feedback_data)

        wait_for_feedback_table(subscription=self.subscription, feedback_records=self.feedback_records)

    def test_17_stats_on_feedback_logging(self):
        self.subscription.feedback_logging.show_table()
        assert_feedback_logging_unstructured_data(subscription=self.subscription, feedback_records=self.feedback_records)

    def test_18_run_quality_monitoring(self):
        run_details = TestAIOpenScaleClient.subscription.quality_monitoring.run()
        assert_quality_entire_run(subscription=TestAIOpenScaleClient.subscription, run_details=run_details)
        TestAIOpenScaleClient.final_run_details = TestAIOpenScaleClient.subscription.quality_monitoring.get_run_details(
            run_uid=run_details['id'])

    def test_19b_get_metrics(self):
        print("\nQuality metrics test: ")
        quality_monitoring_metrics = self.subscription.quality_monitoring.get_metrics()
        data_mart_quality_metrics = self.ai_client.data_mart.get_deployment_metrics(metric_type='quality')
        assert_get_quality_metrics(self.final_run_details, data_mart_quality_metrics, quality_monitoring_metrics)

        quality_monitoring_details = TestAIOpenScaleClient.subscription.quality_monitoring.get_details()
        assert_quality_metrics_binary_model(data_mart_quality_metrics, quality_monitoring_details, subscription_uid=self.subscription_uid)

    def test_19_stats_on_quality_monitoring_table(self):
        self.subscription.quality_monitoring.print_table_schema()
        self.subscription.quality_monitoring.show_table()
        self.subscription.quality_monitoring.show_table(limit=None)
        self.subscription.quality_monitoring.describe_table()

        quality_monitoring_table = self.subscription.quality_monitoring.get_table_content()
        assert_quality_monitoring_pandas_table_content(pandas_table_content=quality_monitoring_table)

        quality_metrics = self.subscription.quality_monitoring.get_table_content(format='python')
        assert_quality_monitoring_python_table_content(python_table_content=quality_metrics)

    @classmethod
    def tearDownClass(cls):
        cls.ai_client.data_mart.subscriptions.delete(
            subscription_uid=cls.subscription_uid
        )


if __name__ == '__main__':
    unittest.main()
